"use strict";

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = buildConfigChain;

var _index = require("../index");

var babel = _interopRequireWildcard(_index);

var _resolve = require("./helpers/resolve");

var _resolve2 = _interopRequireDefault(_resolve);

var _json = require("json5");

var _json2 = _interopRequireDefault(_json);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _micromatch = require("micromatch");

var _micromatch2 = _interopRequireDefault(_micromatch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var existsCache = {};
var jsonCache = {};

var BABELRC_FILENAME = ".babelrc";
var BABELRC_JS_FILENAME = ".babelrc.js";
var PACKAGE_FILENAME = "package.json";
var BABELIGNORE_FILENAME = ".babelignore";

function exists(filename) {
  var cached = existsCache[filename];
  if (cached == null) {
    return existsCache[filename] = _fs2.default.existsSync(filename);
  } else {
    return cached;
  }
}

function buildConfigChain() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var filename = opts.filename ? _path2.default.resolve(opts.filename) : null;
  var builder = new ConfigChainBuilder(filename);

  try {
    builder.mergeConfig({
      type: "arguments",
      options: opts,
      alias: "base",
      dirname: process.cwd()
    });

    if (opts.babelrc !== false) {
      builder.findConfigs(filename);
    }
  } catch (e) {
    if (e.code !== "BABEL_IGNORED_FILE") throw e;

    return null;
  }

  return builder.configs.reverse();
}

var ConfigChainBuilder = function () {
  function ConfigChainBuilder(filename) {
    _classCallCheck(this, ConfigChainBuilder);

    this.resolvedConfigs = [];
    this.configs = [];
    this.filename = filename;
    this.possibleDirs = null;
  }

  ConfigChainBuilder.prototype.shouldIgnore = function shouldIgnore(ignore, only, dirname) {
    if (!this.filename) return false;

    if (ignore) {
      if (!Array.isArray(ignore)) {
        throw new Error(".ignore should be an array, was " + JSON.stringify(ignore));
      }

      for (var _iterator = ignore, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var pattern = _ref;

        if (this.matchesPattern(pattern, dirname)) return true;
      }
    }

    if (only) {
      if (!Array.isArray(only)) {
        throw new Error(".only should be an array, was " + JSON.stringify(only));
      }

      for (var _iterator2 = only, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray2) {
          if (_i2 >= _iterator2.length) break;
          _ref2 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) break;
          _ref2 = _i2.value;
        }

        var _pattern = _ref2;

        if (this.matchesPattern(_pattern, dirname)) return false;
      }
      return true;
    }

    return false;
  };

  ConfigChainBuilder.prototype.matchesPattern = function matchesPattern(pattern, dirname) {
    if (typeof pattern === "string") {
      if (!this.possibleDirs) {
        this.possibleDirs = [];

        if (this.filename) {
          this.possibleDirs.push(this.filename);

          var current = this.filename;
          while (true) {
            var previous = current;
            current = _path2.default.dirname(current);
            if (previous === current) break;

            this.possibleDirs.push(current);
          }
        }
      }

      return this.possibleDirs.some(_micromatch2.default.filter(_path2.default.resolve(dirname, pattern), {
        nocase: true,
        nonegate: true
      }));
    } else if (typeof pattern === "function") {
      return pattern(this.filename);
    } else {
      return pattern.test(this.filename);
    }
  };

  ConfigChainBuilder.prototype.errorMultipleConfigs = function errorMultipleConfigs(loc1, loc2) {
    throw new Error("Multiple configuration files found. Please remove one:\n- " + loc1 + "\n- " + loc2);
  };

  ConfigChainBuilder.prototype.findConfigs = function findConfigs(loc) {
    var _this = this;

    if (!loc) return;

    if (!_path2.default.isAbsolute(loc)) {
      loc = _path2.default.join(process.cwd(), loc);
    }

    var foundConfig = false;
    var foundIgnore = false;

    while (loc !== (loc = _path2.default.dirname(loc))) {
      if (!foundIgnore) {
        var ignoreLoc = _path2.default.join(loc, BABELIGNORE_FILENAME);
        if (exists(ignoreLoc)) {
          this.addIgnoreConfig(ignoreLoc);
          foundIgnore = true;
        }
      }

      if (!foundConfig) {
        (function () {
          var configLoc = _path2.default.join(loc, BABELRC_FILENAME);
          var configJSLoc = _path2.default.join(loc, BABELRC_JS_FILENAME);
          var pkgLoc = _path2.default.join(loc, PACKAGE_FILENAME);
          var configLocs = [configLoc, configJSLoc, pkgLoc];
          var foundConfigs = configLocs.reduce(function (arr, config) {
            if (exists(config)) {
              var configAdded = config === pkgLoc ? _this.addConfig(config, "babel", JSON) : _this.addConfig(config);

              if (configAdded && arr.length) {
                _this.errorMultipleConfigs(arr.pop(), config);
              }

              arr.push(config);
            }

            return arr;
          }, []);

          foundConfig = !!foundConfigs.length;
        })();
      }

      if (foundIgnore && foundConfig) return;
    }
  };

  ConfigChainBuilder.prototype.addIgnoreConfig = function addIgnoreConfig(loc) {
    var file = _fs2.default.readFileSync(loc, "utf8");
    var lines = file.split("\n");

    lines = lines.map(function (line) {
      return line.replace(/#(.*?)$/, "").trim();
    }).filter(function (line) {
      return !!line;
    });

    if (lines.length) {
      this.mergeConfig({
        type: "options",
        options: { ignore: lines },
        alias: loc,
        dirname: _path2.default.dirname(loc)
      });
    }
  };

  ConfigChainBuilder.prototype.addConfig = function addConfig(loc, key) {
    var json = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _json2.default;

    if (this.resolvedConfigs.indexOf(loc) >= 0) {
      return false;
    }

    this.resolvedConfigs.push(loc);

    var options = void 0;
    if (_path2.default.extname(loc) === ".js") {
      try {
        var configModule = require(loc);
        options = configModule && configModule.__esModule ? configModule.default : configModule;
      } catch (err) {
        err.message = loc + ": Error while loading config - " + err.message;
        throw err;
      }

      if (!options || (typeof options === "undefined" ? "undefined" : _typeof(options)) !== "object") {
        throw new Error("Configuration should be an exported JavaScript object.");
      }
    } else {
      var content = _fs2.default.readFileSync(loc, "utf8");
      try {
        options = jsonCache[content] = jsonCache[content] || json.parse(content);
      } catch (err) {
        err.message = loc + ": Error while parsing JSON - " + err.message;
        throw err;
      }
    }

    if (key) {
      if (!options[key]) {
        return false;
      }

      options = options[key];
    }

    this.mergeConfig({
      type: "options",
      options: options,
      alias: loc,
      dirname: _path2.default.dirname(loc)
    });

    return !!options;
  };

  ConfigChainBuilder.prototype.mergeConfig = function mergeConfig(_ref3) {
    var type = _ref3.type,
        options = _ref3.options,
        alias = _ref3.alias,
        loc = _ref3.loc,
        dirname = _ref3.dirname;

    if (!options) {
      return false;
    }

    if (this.filename && this.shouldIgnore(options.ignore, options.only, dirname)) {
      throw Object.assign(new Error("This file has been ignored."), { code: "BABEL_IGNORED_FILE" });
    }

    options = Object.assign({}, options);

    loc = loc || alias;

    var envKey = babel.getEnv();
    if (options.env) {
      var envOpts = options.env[envKey];
      delete options.env;

      this.mergeConfig({
        type: type,
        options: envOpts,
        alias: alias + ".env." + envKey,
        dirname: dirname
      });
    }

    this.configs.push({
      type: type,
      options: options,
      alias: alias,
      loc: loc,
      dirname: dirname
    });

    if (options.extends) {
      var extendsLoc = (0, _resolve2.default)(options.extends, dirname);
      if (extendsLoc) {
        this.addConfig(extendsLoc);
      } else {
        throw new Error("Couldn't resolve extends clause of " + options.extends + " in " + alias);
      }
      delete options.extends;
    }
  };

  return ConfigChainBuilder;
}();