"use strict";

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _index = require("../index");

var context = _interopRequireWildcard(_index);

var _plugin2 = require("./plugin");

var _plugin3 = _interopRequireDefault(_plugin2);

var _babelMessages = require("babel-messages");

var messages = _interopRequireWildcard(_babelMessages);

var _resolve = require("./helpers/resolve");

var _resolve2 = _interopRequireDefault(_resolve);

var _resolvePlugin = require("./helpers/resolve-plugin");

var _resolvePlugin2 = _interopRequireDefault(_resolvePlugin);

var _resolvePreset = require("./helpers/resolve-preset");

var _resolvePreset2 = _interopRequireDefault(_resolvePreset);

var _defaults = require("lodash/defaults");

var _defaults2 = _interopRequireDefault(_defaults);

var _cloneDeepWith = require("lodash/cloneDeepWith");

var _cloneDeepWith2 = _interopRequireDefault(_cloneDeepWith);

var _merge = require("./helpers/merge");

var _merge2 = _interopRequireDefault(_merge);

var _removed = require("./removed");

var _removed2 = _interopRequireDefault(_removed);

var _buildConfigChain = require("./build-config-chain");

var _buildConfigChain2 = _interopRequireDefault(_buildConfigChain);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var optionNames = new Set(["filename", "filenameRelative", "inputSourceMap", "env", "mode", "retainLines", "highlightCode", "suppressDeprecationMessages", "presets", "plugins", "ignore", "only", "code", "metadata", "ast", "extends", "comments", "shouldPrintComment", "wrapPluginVisitorMethod", "compact", "minified", "sourceMaps", "sourceMapTarget", "sourceFileName", "sourceRoot", "babelrc", "sourceType", "auxiliaryCommentBefore", "auxiliaryCommentAfter", "resolveModuleSource", "getModuleId", "moduleRoot", "moduleIds", "moduleId", "passPerPreset", "parserOpts", "generatorOpts"]);

var OptionManager = function () {
  function OptionManager() {
    _classCallCheck(this, OptionManager);

    this.resolvedConfigs = [];
    this.options = OptionManager.createBareOptions();
  }

  OptionManager.memoisePluginContainer = function memoisePluginContainer(fn, loc, i, alias) {
    for (var _iterator = OptionManager.memoisedPlugins, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var cache = _ref;

      if (cache.container === fn) return cache.plugin;
    }

    var obj = void 0;

    if (typeof fn === "function") {
      obj = fn(context);
    } else {
      obj = fn;
    }

    if ((typeof obj === "undefined" ? "undefined" : _typeof(obj)) === "object") {
      var _plugin = new _plugin3.default(obj, alias);
      OptionManager.memoisedPlugins.push({
        container: fn,
        plugin: _plugin
      });
      return _plugin;
    } else {
      throw new TypeError(messages.get("pluginNotObject", loc, i, typeof obj === "undefined" ? "undefined" : _typeof(obj)) + loc + i);
    }
  };

  OptionManager.createBareOptions = function createBareOptions() {
    return {
      sourceType: "module",
      babelrc: true,
      filename: "unknown",
      code: true,
      metadata: true,
      ast: true,
      comments: true,
      compact: "auto",
      highlightCode: true
    };
  };

  OptionManager.normalisePlugin = function normalisePlugin(plugin, loc, i, alias) {
    plugin = plugin.__esModule ? plugin.default : plugin;

    if (!(plugin instanceof _plugin3.default)) {
      if (typeof plugin === "function" || (typeof plugin === "undefined" ? "undefined" : _typeof(plugin)) === "object") {
        plugin = OptionManager.memoisePluginContainer(plugin, loc, i, alias);
      } else {
        throw new TypeError(messages.get("pluginNotFunction", loc, i, typeof plugin === "undefined" ? "undefined" : _typeof(plugin)));
      }
    }

    plugin.init(loc, i);

    return plugin;
  };

  OptionManager.normalisePlugins = function normalisePlugins(loc, dirname, plugins) {
    return plugins.map(function (val, i) {
      var plugin = void 0,
          options = void 0;

      if (!val) {
        throw new TypeError("Falsy value found in plugins");
      }

      if (Array.isArray(val)) {
        plugin = val[0];
        options = val[1];
      } else {
        plugin = val;
      }

      var alias = typeof plugin === "string" ? plugin : loc + "$" + i;

      if (typeof plugin === "string") {
        var pluginLoc = (0, _resolvePlugin2.default)(plugin, dirname);
        if (pluginLoc) {
          plugin = require(pluginLoc);
        } else {
          throw new ReferenceError(messages.get("pluginUnknown", plugin, loc, i, dirname));
        }
      }

      plugin = OptionManager.normalisePlugin(plugin, loc, i, alias);

      return [plugin, options];
    });
  };

  OptionManager.prototype.mergeOptions = function mergeOptions(_ref2) {
    var _this = this;

    var type = _ref2.type,
        rawOpts = _ref2.options,
        extendingOpts = _ref2.extending,
        alias = _ref2.alias,
        loc = _ref2.loc,
        dirname = _ref2.dirname;

    alias = alias || "foreign";
    if (!rawOpts) return;

    if ((typeof rawOpts === "undefined" ? "undefined" : _typeof(rawOpts)) !== "object" || Array.isArray(rawOpts)) {
      throw new TypeError("Invalid options type for " + alias);
    }

    var opts = (0, _cloneDeepWith2.default)(rawOpts, function (val) {
      if (val instanceof _plugin3.default) {
        return val;
      }
    });

    dirname = dirname || process.cwd();
    loc = loc || alias;

    if (type !== "arguments") {
      if (opts.filename !== undefined) {
        throw new Error(alias + ".filename is only allowed as a root argument");
      }

      if (opts.babelrc !== undefined) {
        throw new Error(alias + ".babelrc is only allowed as a root argument");
      }
    }

    if (type === "preset") {
      if (opts.only !== undefined) throw new Error(alias + ".only is not supported in a preset");
      if (opts.ignore !== undefined) throw new Error(alias + ".ignore is not supported in a preset");
      if (opts.extends !== undefined) throw new Error(alias + ".extends is not supported in a preset");
      if (opts.env !== undefined) throw new Error(alias + ".env is not supported in a preset");
    }

    if (opts.sourceMap !== undefined) {
      if (opts.sourceMaps !== undefined) {
        throw new Error("Both " + alias + ".sourceMap and .sourceMaps have been set");
      }

      opts.sourceMaps = opts.sourceMap;
      delete opts.sourceMap;
    }

    for (var _key in opts) {
      if (!optionNames.has(_key)) {
        if (_removed2.default[_key]) {
          throw new ReferenceError("Using removed Babel 5 option: " + alias + "." + _key + " - " + _removed2.default[_key].message);
        } else {
          var unknownOptErr = "Unknown option: " + alias + "." + _key + ". Check out http://babeljs.io/docs/usage/options/ for more information about options.";

          throw new ReferenceError(unknownOptErr);
        }
      }
    }

    if (opts.parserOpts && typeof opts.parserOpts.parser === "string") {
      var parser = (0, _resolve2.default)(opts.parserOpts.parser, dirname);
      if (parser) {
        opts.parserOpts.parser = require(parser).parse;
      } else {
        throw new Error("Couldn't find parser " + opts.parserOpts.parser + " with \"parse\" method " + ("relative to directory " + dirname));
      }
    }

    if (opts.generatorOpts && typeof opts.generatorOpts.generator === "string") {
      var generator = (0, _resolve2.default)(opts.generatorOpts.generator, dirname);
      if (generator) {
        opts.generatorOpts.generator = require(generator).print;
      } else {
        throw new Error("Couldn't find generator " + opts.generatorOpts.generator + " with \"print\" method " + ("relative to directory " + dirname));
      }
    }

    if (opts.plugins) {
      if (!Array.isArray(rawOpts.plugins)) throw new Error(alias + ".plugins should be an array");

      opts.plugins = OptionManager.normalisePlugins(loc, dirname, opts.plugins);
    }

    if (opts.presets) {
      if (!Array.isArray(rawOpts.presets)) throw new Error(alias + ".presets should be an array");

      opts.presets = this.resolvePresets(opts.presets, dirname, function (preset, presetLoc) {
        _this.mergeOptions({
          type: "preset",
          options: preset,

          extending: opts.passPerPreset ? preset : null,
          alias: presetLoc,
          loc: presetLoc,
          dirname: dirname
        });
      });

      if (!opts.passPerPreset) delete opts.presets;
    }

    if (rawOpts === extendingOpts) {
      Object.assign(extendingOpts, opts);
    } else {
      (0, _merge2.default)(extendingOpts || this.options, opts);
    }
  };

  OptionManager.prototype.resolvePresets = function resolvePresets(presets, dirname, onResolve) {
    var _this2 = this;

    return presets.map(function (preset) {
      var options = void 0;
      if (Array.isArray(preset)) {
        if (preset.length > 2) {
          throw new Error("Unexpected extra options " + JSON.stringify(preset.slice(2)) + " passed to preset.");
        }

        var _preset = preset;
        preset = _preset[0];
        options = _preset[1];
      }

      var presetLoc = void 0;
      try {
        if (typeof preset === "string") {
          presetLoc = (0, _resolvePreset2.default)(preset, dirname);

          if (!presetLoc) {
            throw new Error("Couldn't find preset " + JSON.stringify(preset) + " relative to directory " + JSON.stringify(dirname));
          }
        }
        var resolvedPreset = _this2.loadPreset(presetLoc || preset, options, { dirname: dirname });

        if (onResolve) onResolve(resolvedPreset, presetLoc);

        return resolvedPreset;
      } catch (e) {
        if (presetLoc) {
          e.message += " (While processing preset: " + JSON.stringify(presetLoc) + ")";
        }
        throw e;
      }
    });
  };

  OptionManager.prototype.loadPreset = function loadPreset(preset, options, meta) {
    var presetFactory = preset;
    if (typeof presetFactory === "string") {
      presetFactory = require(presetFactory);
    }

    if ((typeof presetFactory === "undefined" ? "undefined" : _typeof(presetFactory)) === "object" && presetFactory.__esModule) {
      if (presetFactory.default) {
        presetFactory = presetFactory.default;
      } else {
        throw new Error("Preset must export a default export when using ES6 modules.");
      }
    }

    if ((typeof presetFactory === "undefined" ? "undefined" : _typeof(presetFactory)) === "object") {
      return presetFactory;
    }

    if (typeof presetFactory !== "function") {
      throw new Error("Unsupported preset format: " + (typeof presetFactory === "undefined" ? "undefined" : _typeof(presetFactory)) + ". Expected preset to return a function.");
    }

    return presetFactory(context, options, meta);
  };

  OptionManager.prototype.init = function init() {
    var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var configChain = (0, _buildConfigChain2.default)(opts);
    if (!configChain) return null;

    try {
      for (var _iterator2 = configChain, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref3;

        if (_isArray2) {
          if (_i2 >= _iterator2.length) break;
          _ref3 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) break;
          _ref3 = _i2.value;
        }

        var config = _ref3;

        this.mergeOptions(config);
      }
    } catch (e) {
      if (!/^\[BABEL\]/.test(e.message)) {
        e.message = "[BABEL] " + (opts.filename || "unknown") + ": " + e.message;
      }

      throw e;
    }

    opts = this.options;

    if (opts.inputSourceMap) {
      opts.sourceMaps = true;
    }

    if (opts.moduleId) {
      opts.moduleIds = true;
    }

    (0, _defaults2.default)(opts, {
      moduleRoot: opts.sourceRoot
    });

    (0, _defaults2.default)(opts, {
      sourceRoot: opts.moduleRoot
    });

    (0, _defaults2.default)(opts, {
      filenameRelative: opts.filename
    });

    var basenameRelative = _path2.default.basename(opts.filenameRelative);

    (0, _defaults2.default)(opts, {
      sourceFileName: basenameRelative,
      sourceMapTarget: basenameRelative
    });

    return opts;
  };

  return OptionManager;
}();

exports.default = OptionManager;


OptionManager.memoisedPlugins = [];